@@ -17,9 +17,12 @@ sys.ps1 = 'OMFIT >>> '
 from numpy.core.multiarray import array as _array
 
 
-def array(object, dtype=None, copy=True, order=None, subok=False, ndmin=0):
+def array(object, dtype=None, *, copy=True, order='K', subok=False, ndmin=0, like=None):
     object = evalExpr(object)
-    return _array(object, dtype=dtype, copy=copy, order=order, subok=subok, ndmin=ndmin)
+    if like is not None:
+        return _array(object, dtype=dtype, copy=copy, order=order, subok=subok, ndmin=ndmin, like=None)
+    else:
+        return _array(object, dtype=dtype, copy=copy, order=order, subok=subok, ndmin=ndmin)
 
 
 array.__doc__ = _array.__doc__
@@ -27,9 +30,19 @@ array.__doc__ = _array.__doc__
 from numpy.core.fromnumeric import asarray as _asarray
 
 
-def asarray(a, dtype=None, order=None):
+def asarray(a, dtype=None, order=None, *, like=None):
     a = evalExpr(a)
-    return _asarray(a, dtype=dtype, order=order)
+    if like is not None:
+        return _asarray(a, dtype=dtype, order=order, like=like)
+    else:
+        try:
+            return _asarray(a, dtype=dtype, order=order)
+        except Exception:
+            print("=!=!=!=")
+            print(a)
+            print(type(a))
+            print("=!=!=!=")
+            raise
 
 
 asarray.__doc__ = _asarray.__doc__
