@@ -15,6 +15,7 @@ except ImportError as _excp:
 from omfit_classes.omfit_namelist import OMFITnamelist
 from omfit_classes.omfit_ascii import OMFITascii
 from omfit_classes.fluxSurface import rz_miller
+from omfit_classes.omfit_eqdsk import OMFITgeqdsk
 from omfit_classes.namelist import NamelistName
 import numpy as np
 import fortranformat
@@ -123,27 +124,27 @@ class OMFITmhdin(OMFITnamelist):
                             kk += 1
                         self['VESSEL'] = np.array(self['VESSEL'].values())
 
-            # create namelist elements
-            if len(self['FC']):
-                self['IN3']['RF'] = self['FC'][:, 0]
-                self['IN3']['ZF'] = self['FC'][:, 1]
-                self['IN3']['WF'] = self['FC'][:, 2]
-                self['IN3']['HF'] = self['FC'][:, 3]
-                self['IN3']['AF'] = self['FC'][:, 4]
-                self['IN3']['AF2'] = self['FC'][:, 5]
-            if len(self['VESSEL']):
-                self['IN3']['RVS'] = self['VESSEL'][:, 0]
-                self['IN3']['ZVS'] = self['VESSEL'][:, 1]
-                self['IN3']['WVS'] = self['VESSEL'][:, 2]
-                self['IN3']['HVS'] = self['VESSEL'][:, 3]
-                self['IN3']['AVS'] = self['VESSEL'][:, 5]
-                self['IN3']['AVS2'] = self['VESSEL'][:, 5]
-            if len(self['OH']):
-                self['IN3']['RE'] = self['OH'][:, 0]
-                self['IN3']['ZE'] = self['OH'][:, 1]
-                self['IN3']['WE'] = self['OH'][:, 2]
-                self['IN3']['HE'] = self['OH'][:, 3]
-                self['IN3']['ECID'] = self['OH'][:, 4]
+                    # create namelist elements
+                    if len(self['FC']):
+                        self['IN3']['RF'] = self['FC'][:, 0]
+                        self['IN3']['ZF'] = self['FC'][:, 1]
+                        self['IN3']['WF'] = self['FC'][:, 2]
+                        self['IN3']['HF'] = self['FC'][:, 3]
+                        self['IN3']['AF'] = self['FC'][:, 4]
+                        self['IN3']['AF2'] = self['FC'][:, 5]
+                    if len(self['VESSEL']):
+                        self['IN3']['RVS'] = self['VESSEL'][:, 0]
+                        self['IN3']['ZVS'] = self['VESSEL'][:, 1]
+                        self['IN3']['WVS'] = self['VESSEL'][:, 2]
+                        self['IN3']['HVS'] = self['VESSEL'][:, 3]
+                        self['IN3']['AVS'] = self['VESSEL'][:, 5]
+                        self['IN3']['AVS2'] = self['VESSEL'][:, 5]
+                    if len(self['OH']):
+                        self['IN3']['RE'] = self['OH'][:, 0]
+                        self['IN3']['ZE'] = self['OH'][:, 1]
+                        self['IN3']['WE'] = self['OH'][:, 2]
+                        self['IN3']['HE'] = self['OH'][:, 3]
+                        self['IN3']['ECID'] = self['OH'][:, 4]
 
         else:
             # initialize namelist
@@ -151,7 +152,7 @@ class OMFITmhdin(OMFITnamelist):
                 for element in ['E', 'F', 'VS']:
                     item = geom.format(element=element)
                     if item not in self['IN3'] and item not in ['AE', 'AE2']:
-                        self['IN3'][item] = np.array([])
+                        self['IN3'][item] = []
 
         if 'FC' in self:
             del self['FC']
@@ -186,9 +187,10 @@ class OMFITmhdin(OMFITnamelist):
             if isinstance(self['IN3'][k], (list, np.ndarray)) and not len(self['IN3'][k]):
                 empty.append(k)
                 del self['IN3'][k]
+
         super().save(*args, **kw)
         for k in list(empty):
-            self['IN3'][k] = np.ndarray([])
+            self['IN3'][k] = []
 
     @staticmethod
     def plot_coil(data, patch_facecolor='lightgray', patch_edgecolor='black', label=None, ax=None):
