@@ -1096,15 +1096,19 @@ else:
 
         default_GACODE_branch = OMFITexpression(
             '''
-version = 'tglf_sat2'
+version = '2022.17'
 server = root['SETTINGS']['REMOTE_SETUP']['serverPicker']
 if is_server(server, ['iris', 'saturn']):
     if version == 'pre_pygacode':
         return_variable = '/dev'
     elif version=='post_pygacode':
         return_variable = '/pygacode'
-    else:
+    elif version=='tglf_sat2':
         return_variable = '/tglf_sat2'
+    elif version=='2022.17':
+        return_variable = '/2022.17'
+elif is_server(server, ['omega']):
+    return_variable = '/2022.17'
 elif is_server(server, 'portal'):
     if version == 'pre_pygacode':
         return_variable = '/unstable'
@@ -1122,7 +1126,6 @@ else:
         return_variable = ''
     else:
         raise ValueError(f'GACODE installation on {server} has not yet been updated to post_gacode')
-
     '''.strip()
         )
 
@@ -1143,8 +1146,8 @@ else:
                         if k in OMFIT[module]['SETTINGS']['REMOTE_SETUP']['iris']:
                             OMFIT[module]['SETTINGS']['REMOTE_SETUP'][item][k] = copy.deepcopy(root['SETTINGS']['REMOTE_SETUP'][item][k])
                 else:
-                    OMFIT[module]['SETTINGS']['REMOTE_SETUP'][item]['executable'] = copy.deepcopy(
-                        root['SETTINGS']['REMOTE_SETUP'][item]['executable']
+                    OMFIT[module]['SETTINGS']['REMOTE_SETUP'][item]['environment'] = copy.deepcopy(
+                        root['SETTINGS']['REMOTE_SETUP'][item]['environment']
                     )
             OMFIT[module]['SETTINGS']['REMOTE_SETUP']['saturn']['executable'] = OMFIT[module]['SETTINGS']['REMOTE_SETUP']['iris'][
                 'executable'
