@@ -181,8 +181,8 @@ class OMFITmars(SortedDict, OMFITobject):
             from cmath import log
 
             m0 = 2
-            I0 = np.int(self[self.sim]['Iratsurf'][0])
-            m1 = np.int(m0 - Mm[0] + 1)
+            I0 = int(self[self.sim]['Iratsurf'][0])
+            m1 = int(m0 - Mm[0] + 1)
             p0 = -np.angle(JM2[I0 - 1, m1 - 1]) + np.pi / 2.0
             f0 = np.exp(1j * p0)
             JM1 = JM1 * f0
@@ -410,8 +410,8 @@ class OMFITmars(SortedDict, OMFITobject):
             xnorm = R0 * 1.0e3  # displacement [mm]
             bnorm = B0 * 1.0e4  # b-field [Gauss]
             cnorm = R0 * B0 / mu0  # current [A]
-            tdnorm = (B0 ** 2) / mu0 / np.pi  # torque density [N/m^2]
-            ttnorm = tdnorm * (R0 ** 3)  # total torque [Nm]
+            tdnorm = (B0**2) / mu0 / np.pi  # torque density [N/m^2]
+            ttnorm = tdnorm * (R0**3)  # total torque [Nm]
             tmnorm = TAUA0 * 1.0e3  # time [ms]
             unit_t = 'ms'
             unit_b = 'Gauss'
@@ -541,16 +541,10 @@ class OMFITmars(SortedDict, OMFITobject):
                 attrs={'Description': 'Real parallel dWk for species = %s, type = %s' % (KP, I)},
             )
             self['DWK_ENERGY_DENSITY']['%s_%s_IMdwppara' % (KP, I)] = xarray.DataArray(
-                blocks[l][:, 5],
-                dims=['s'],
-                coords={'s': CSM},
-                attrs={'Description': f'Imag parallel dWk for species = {KP}, type = {I}'},
+                blocks[l][:, 5], dims=['s'], coords={'s': CSM}, attrs={'Description': f'Imag parallel dWk for species = {KP}, type = {I}'}
             )
             self['DWK_ENERGY_DENSITY']['%s_%s_REdwpperp' % (KP, I)] = xarray.DataArray(
-                blocks[l][:, 6],
-                dims=['s'],
-                coords={'s': CSM},
-                attrs={'Description': 'Real perp. dWk for specie = %s, type = %s' % (KP, I)},
+                blocks[l][:, 6], dims=['s'], coords={'s': CSM}, attrs={'Description': 'Real perp. dWk for specie = %s, type = %s' % (KP, I)}
             )
             self['DWK_ENERGY_DENSITY']['%s_%s_IMdwpperp' % (KP, I)] = xarray.DataArray(
                 blocks[l][:, 7],
@@ -559,16 +553,10 @@ class OMFITmars(SortedDict, OMFITobject):
                 attrs={'Description': 'Imag. perp. dWk for specie = %s, type = %s' % (KP, I)},
             )
             self['DWK_ENERGY_DENSITY']['%s_%s_REdwk' % (KP, I)] = xarray.DataArray(
-                blocks[l][:, 8],
-                dims=['s'],
-                coords={'s': CSM},
-                attrs={'Description': 'Real total dWk for specie = %s, type = %s' % (KP, I)},
+                blocks[l][:, 8], dims=['s'], coords={'s': CSM}, attrs={'Description': 'Real total dWk for specie = %s, type = %s' % (KP, I)}
             )
             self['DWK_ENERGY_DENSITY']['%s_%s_IMdwk' % (KP, I)] = xarray.DataArray(
-                blocks[l][:, 9],
-                dims=['s'],
-                coords={'s': CSM},
-                attrs={'Description': 'Imag total dWk for specie = %s, type = %s' % (KP, I)},
+                blocks[l][:, 9], dims=['s'], coords={'s': CSM}, attrs={'Description': 'Imag total dWk for specie = %s, type = %s' % (KP, I)}
             )
         return
 
