@@ -18,8 +18,10 @@ except ImportError as _excp:
 
 import numpy as np
 import copy
+from omfit_classes.omfit_ascii import OMFITascii
 from omfit_classes.omfit_mds import OMFITmds, OMFITmdsValue
 from omfit_classes.utils_base import compare_version
+from omfit_classes.omfit_omas_utils import ensure_consistent_experiment_info
 
 try:
     import omas
@@ -28,7 +30,7 @@ try:
 except Exception:
     imas_version = '0.0'
 
-__all__ = []
+__all__ = ['OMFITd3dcompfile']
 
 
 # Decorators
@@ -1127,6 +1129,9 @@ def setup_langmuir_probes_hardware_description_d3d(ods, shot):
         printe('ABORTED setup_langmuir_probes_hardware_description_d3d due to old IMAS version.')
         return {}
 
+    ods['dataset_description.data_entry'].setdefault('machine', 'DIII-D')
+    ods['dataset_description.data_entry'].setdefault('pulse', shot)
+
     tdi = r'GETNCI("\\langmuir::top.probe_*.r", "LENGTH")'
     # "LENGTH" is the size of the data, I think (in bits?). Single scalars seem to be length 12.
     printq('Setting up Langmuir probes hardware description, shot {}; checking availability, TDI={}'.format(shot, tdi))
@@ -1157,3 +1162,332 @@ def setup_langmuir_probes_hardware_description_d3d(ods, shot):
             else:
                 printq('Probe i={i:}, j={j:}, r={r:} failed the check with chk={chk:}'.format(**locals()))
     return {}
+
+
+@make_available
+def find_active_d3d_probes(shot, allowed_probes=None):
+    """
+    Serves LP functions by identifying active probes (those that have actual data saved) for a given shot
+
+    Sorry, I couldn't figure out how to do this with a server-side loop over all
+    the probes, so we have to loop MDS calls on the client side. At least I
+    resampled to speed that part up.
+
+    This could be a lot faster if I could figure out how to make the GETNCI
+    commands work on records of array signals.
+
+    :param shot: int
+
+    :param allowed_probes: int array
+        Restrict the search to a certain range of probe numbers to speed things up
+        These are the numbers of storage trees in MDSplus, not the physical probe numbers
+
+    :return: list of ints
+    """
+    print(f'Searching for active DIII-D probes in shot {shot}...')
+    omv_kw = dict(server='DIII-D', shot=shot, treename='langmuir')
+    lp_paths_unfiltered = OMFITmdsValue(TDI=r'GETNCI("\\TOP.PROBE_*.TEMP", "MINPATH")', **omv_kw).data()
+    lp_paths = [lpp.strip() for lpp in lp_paths_unfiltered if re.match('[0-9]', lpp.strip().split(':')[0].split('_')[-1])]
+    print(f'{len(lp_paths)} possible probes found')
+    lp_nums = np.array([int(lpp.strip().split(':')[0].split('_')[-1]) for lpp in lp_paths])
+    idx = lp_nums.argsort()
+    lp_nums = lp_nums[idx]
+    lp_paths = np.array(lp_paths)[idx]
+    if allowed_probes is None:
+        acceptable_probe_numbers = lp_nums
+        acceptable_paths = lp_paths
+    else:
+        acceptable_probe_numbers = np.array([lpn for lpn in lp_nums if lpn in allowed_probes])
+        acceptable_paths = np.array([lp_paths[i] for i in range(len(lp_nums)) if lp_nums[i] in allowed_probes])
+        print(f'{len(acceptable_paths)} candidates remain after limiting search to only some specific probes.')
+    valid = [None] * len(acceptable_paths)
+    OMFIT['acceptable_paths'] = acceptable_paths
+    OMFIT['acceptable_probe_numbers'] = acceptable_probe_numbers
+    OMFIT['lp_nums'] = lp_nums
+    OMFIT['lp_paths'] = lp_paths
+
+    i = wd = tc = 0
+    for i in range(len(acceptable_paths)):
+        ascii_progress_bar(
+            i,
+            0,
+            len(acceptable_paths),
+            mess=f'Checking DIII-D#{shot} LP probes for valid data {lp_paths[i]} valid={valid[i]}; '
+            f'probes with valid data so far = {wd}/{tc}',
+        )
+        m = OMFITmdsValue(TDI=f'resample({acceptable_paths[i]}, 0, 1, 1)', **omv_kw)
+        valid[i] = m.check() and np.any(m.data() > 0)
+        wd = np.sum(np.array(valid).astype(bool))
+        tc = len([a for a in valid if a is not None])
+    ascii_progress_bar(
+        i + 1,
+        0,
+        len(acceptable_paths),
+        mess=f'Checked DIII-D#{shot} LP probes for valid data and found {wd} out of {tc} probes have valid data.',
+    )
+    print()
+    return acceptable_probe_numbers[np.array(valid).astype(bool)]
+
+
+@make_available_if(compare_version(imas_version, '3.25.0') >= 0)  # Might actually be 3.24.X. Close enough for now.
+def load_data_langmuir_probes_d3d(
+    ods, shot, probes=None, allowed_probes=None, tstart=0, tend=0, dt=0.0002, overwrite=False, quantities=None
+):
+    """
+    Downloads LP probe data from MDSplus and loads them to the ODS
+
+    :param ods: ODS instance
+
+    :param shot: int
+
+    :param probes: int array-like [optional]
+        Integer array of DIII-D probe numbers.
+        If not provided, find_active_d3d_probes() will be used.
+
+    :param allowed_probes: int array-like [optional]
+        Passed to find_active_d3d_probes(), if applicable.
+        Improves speed by limiting search to a specific range of probe numbers.
+
+    :param tstart: float
+        Time to start resample (s)
+
+    :param tend: float
+        Time to end resample (s)
+        Set to <= tstart to disable resample
+        Server-side resampling does not work when time does not increase
+        monotonically, which is a typical problem for DIII-D data. Resampling is
+        not recommended for DIII-D.
+
+    :param dt: float
+        Resample interval (s)
+        Set to 0 to disable resample
+        Server-side resampling does not work when time does not increase
+        monotonically, which is a typical problem for DIII-D data. Resampling is
+        not recommended for DIII-D.
+
+    :param overwrite: bool
+        Download and write data even if they already are present in the ODS.
+
+    :param quantities: list of strings [optional]
+        List of quantities to gather. None to gather all available. Options are:
+        ion_saturation_current, heat_flux_parallel, n_e, t_e', surface_area_effective,
+        v_floating, and b_field_angle
+
+    :return: ODS instance
+        The data are added in-place, so catching the return is probably unnecessary.
+    """
+
+    tf = 1e3  # DIII-D stores LP data in ms, not s
+
+    # Make sure the ODS is for the right device/shot
+    device = 'DIII-D'
+    ensure_consistent_experiment_info(ods, device, shot)
+
+    # Make sure the ODS already has probe positions in it
+    hw_ready = True
+    try:
+        if not is_numeric(ods['langmuir_probes.embedded[0].position.r']):
+            hw_ready = False
+        if 'name' not in ods['langmuir_probes.embedded.0']:
+            hw_ready = False
+    except Exception:
+        hw_ready = False
+    if not hw_ready:
+        setup_langmuir_probes_hardware_description_d3d(ods, shot)
+
+    # Select probes to gather
+    probes = probes or find_active_d3d_probes(shot, allowed_probes=allowed_probes)
+
+    do_resample = (dt > 0) and (tend > tstart)
+    if do_resample:
+        printe('DIII-D often has 0s @ end of timebase for LP data in MDSplus, breaking server-side resampling.')
+        printw('Recommended: abort and set tend=tstart or dt=0 to disable resampling.')
+
+    available_quantities = dict(
+        ion_saturation_current='ISAT',
+        heat_flux_parallel='HEATFLUX',
+        n_e='DENS',
+        t_e='TEMP',
+        surface_area_effective='AREA',
+        v_floating='POT',  # or is it 'v_plasma'?
+        b_field_angle='ANGLE',
+    )
+    if quantities is None:
+        quantities = available_quantities
+    else:
+        quantities = {k: v for k, v in available_quantities.items() if k in quantities}
+    factors = dict(
+        ion_saturation_current=1,  # A to A
+        heat_flux_parallel=1e7,  # kW cm^-2 to W m^-2
+        n_e=1e6,  # cm^-3 to m^-3
+        t_e=1,  # eV to eV
+        surface_area_effective=1e-4,  # cm^2 to m^2
+        v_floating=1,  # V to V
+        v_plasma=1,  # V to V
+        b_field_angle=np.pi / 180.0,  # deg to rad
+    )
+    has_err = ['t_e', 'n_e', 'v_floating']
+    i = 0
+    for i, probe in enumerate(probes):
+        p_idx = probe - 1
+        tdi_base = rf'\TOP.PROBE_{probe:03d}'
+
+        if not overwrite:
+            try:
+                _ = ods['langmuir_probes.embedded'][i]['time']
+                _ = ods['langmuir_probes.embedded'][i]['ion_saturation_current.data']
+            except Exception:
+                pass
+            else:
+                printd(f'Skipping probe {probe} with index {p_idx} because it already has data', topic='omfit_omas_d3d')
+                continue
+
+        for j, ods_loc in enumerate(quantities):
+            quantity = quantities[ods_loc]
+            tdi = f'{tdi_base}.{quantity}'
+            ascii_progress_bar(
+                i * len(quantities) + j, 0, len(probes) * len(quantities), mess=f'Loading {device}#{shot} LP probe data ({probe}, {tdi})'
+            )
+            f = factors.get(ods_loc, 1.0)
+            if ods_loc in has_err:
+                tdi_err = f'{tdi}_ERR'
+            else:
+                tdi_err = None
+            if do_resample:
+                tdi = f'resample({tdi}, {tstart*tf}, {tend*tf}, {dt*tf})'
+                if tdi_err is not None:
+                    tdi_err = f'resample({tdi_err}, {tstart*tf}, {tend*tf}, {dt*tf})'
+
+            m = OMFITmdsValue(server=device, shot=shot, treename='LANGMUIR', TDI=tdi)
+            if tdi_err is not None:
+                m_err = OMFITmdsValue(server=device, shot=shot, treename='LANGMUIR', TDI=tdi_err)
+            else:
+                m_err = None
+
+            if m.check():
+                ods['langmuir_probes.embedded'][i]['time'] = m.dim_of(0) / tf
+                if m_err is not None:
+                    ods['langmuir_probes.embedded'][i][ods_loc]['data'] = uarray(m.data() * f, abs(m_err.data()) * f)
+                else:
+                    ods['langmuir_probes.embedded'][i][ods_loc]['data'] = m.data() * f  # Just nominal values
+            else:
+                printd(
+                    f'Probe {i} (#{p_idx:03d}) does not appear to have good {quantity} ({ods_loc}) data. Skipping...',
+                    topic='omfit_omas_d3d',
+                )
+                printd(f'    TDI = {tdi}', topic='omfit_omas_d3d')
+
+    ascii_progress_bar(i + 1, 0, len(probes), mess=f'Loaded {device}#{shot} LP probe data')
+    if do_resample:
+        printe('DIII-D often has 0s @ end of timebase for LP data in MDSplus, breaking server-side resampling.')
+        printw('Recommended: check your data and repeat with dt=0 or tend=tstart if you find problems.')
+
+    return ods
+
+
+class OMFITd3dcompfile(SortedDict, OMFITascii):
+    """
+    OMFIT class to read DIII-D compensation files such as btcomp.dat ccomp.dat and icomp.dat
+    """
+
+    def __init__(self, filename, use_leading_comma=None, **kw):
+        r"""
+        OMFIT class to parse DIII-D MHD device files
+
+        :param filename: filename
+
+        :param \**kw: arguments passed to __init__ of OMFITascii
+        """
+        OMFITascii.__init__(self, filename, **kw)
+        SortedDict.__init__(self)
+        self.dynaLoad = True
+
+    @dynaLoad
+    def load(self):
+        self.clear()
+        with open(self.filename, 'r') as f:
+            lines = f.readlines()
+
+        for line in lines:
+            linesplit = line.split()
+
+            try:
+                compshot = int(eval(linesplit[0]))
+                self[compshot] = {}
+                for compsig in linesplit[1:]:
+                    self[compshot][compsig] = {}
+
+            except Exception:
+                sig = linesplit[0][1:].strip()
+                comps = [float(i) for i in linesplit[2:]]
+                for comp, compsig in zip(comps, self[compshot]):
+                    self[compshot][compsig][sig] = comp
+
+        return self
+
+
+class OMFITd3dfitweight(SortedDict, OMFITascii):
+    """
+    OMFIT class to read DIII-D fitweight file
+    """
+
+    def __init__(self, filename, use_leading_comma=None, **kw):
+        r"""
+        OMFIT class to parse DIII-D device files
+
+        :param filename: filename
+
+        :param \**kw: arguments passed to __init__ of OMFITascii
+        """
+        OMFITascii.__init__(self, filename, **kw)
+        SortedDict.__init__(self)
+        self.dynaLoad = True
+
+    @dynaLoad
+    def load(self):
+        self.clear()
+
+        magpri67 = 29
+        magpri322 = 31
+        magprirdp = 8
+        magudom = 5
+        maglds = 3
+        nsilds = 3
+        nsilol = 41
+
+        with open(self.filename, 'r') as f:
+            data = f.read()
+
+        data = data.strip().split()
+
+        for i in data:
+            ifloat = float(i)
+            if ifloat > 100:
+                ishot = ifloat
+                self[ifloat] = []
+            else:
+                self[ishot].append(ifloat)
+
+        for irshot in self:
+            if irshot < 124985:
+                mloop = nsilol
+            else:
+                mloop = nsilol + nsilds
+
+            if irshot < 59350:
+                mprobe = magpri67
+            elif irshot < 91000:
+                mprobe = magpri67 + magpri322
+            elif irshot < 100771:
+                mprobe = magpri67 + magpri322 + magprirdp
+            elif irshot < 124985:
+                mprobe = magpri67 + magpri322 + magprirdp + magudom
+            else:
+                mprobe = magpri67 + magpri322 + magprirdp + magudom + maglds
+            fwtmp2 = self[irshot][mloop : mloop + mprobe]
+            fwtsi = self[irshot][0:mloop]
+            self[irshot] = {}
+            self[irshot]['fwtmp2'] = fwtmp2
+            self[irshot]['fwtsi'] = fwtsi
+
+        return self
