@@ -295,7 +295,8 @@ class OMFITtranspData(SortedDict):
             if fig.axes:
                 axes = fig.axes[0]
             else:
-                axes = fig.add_subplot(111)
+                pyplot.figure(num=fig.number)
+                axes = pyplot.subplot(111)
         else:
             fig = pyplot.gcf()
             axes = pyplot.gca()
@@ -510,8 +511,8 @@ class OMFITtranspMultigraph(SortedDict):
                 toolbar = NavigationToolbar2(canvas, tab)
                 toolbar.update()
                 canvas._tkcanvas.pack(side='top', fill='both', expand=1)
-                axx = fig.add_subplot(211)
-                axy = fig.add_subplot(212)
+                axx = fig.add_subplot(211)  # OKadd
+                axy = fig.add_subplot(212)  # OKadd
                 tab.fig = fig
 
                 def update_lines(tab, event=None):
